/**
  *  \file planetscentral/dbclient/dbhash.h
  */
#ifndef C2SERVER_PLANETSCENTRAL_DBCLIENT_DBHASH_H
#define C2SERVER_PLANETSCENTRAL_DBCLIENT_DBHASH_H

#include "planetscentral/dbclient/dbkey.h"
#include "planetscentral/dbclient/connection.h"

class DbHashIntField;
class DbHashStringField;
class DbSort;

class DbHash : public DbKey {
 public:
    DbHash(DbClient& client, string_t name);

    bool     removeField(string_t name);
    bool     hasField(string_t name);
    string_t getStringField(string_t name);
    int32_t  getIntField(string_t name);
    IntValue* getFieldRaw(string_t name);

    int32_t  addToField(string_t name, int32_t n);
    int32_t  size();
    bool     setField(string_t name, string_t value);
    bool     setField(string_t name, int32_t value);
    bool     setFieldRaw(string_t name, IntValue* value);
    void     setFields(const StringList_t& vec);

    void     getAll(StringList_t& vec);
    void     getAll(IntList_t& vec);
    void     setAll(const StringList_t& vec);
    void     keys(StringList_t& vec);
    IntValue* getFields(const StringList_t& vec);

    DbHashIntField intField(string_t name);
    DbHashStringField stringField(string_t name);
};

class DbHashField {
    friend class DbSort;
 public:
    DbHashField(DbHash& h, const string_t& name);
    ~DbHashField();
    bool exists();
    bool remove();
    IntValue* getRaw();

 protected:
    DbHash h;
    string_t name;
};

class DbHashIntField : public DbHashField {
 public:
    DbHashIntField(DbHash& h, const string_t& name);

    int32_t operator++();
    int32_t operator+=(int32_t incr);
    int32_t operator--();
    int32_t operator-=(int32_t incr);

    bool set(int32_t value);
    int32_t get();
};

class DbHashStringField : public DbHashField {
 public:
    DbHashStringField(DbHash& h, const string_t& name);

    bool set(string_t value);
    string_t get();
};


//?HKEYS key               list<string>                            all keys
//?HMGET key f1 f2...      list<string>                            HGET key f1; HGET key f2; ...
//?HVALS key               list<string>                            all values

#endif
