/**
  *  \file planetscentral/talk/url.cc
  *  \brief Talk: URLs
  */

#include "planetscentral/talk/url.h"

/** Simplify a topic name for inclusion in an URL. */
string_t
simplifyTopic(string_t s)
{
    string_t result;
    bool needsp = false;
    for (string_t::size_type i = 0; i < s.size(); ++i) {
        char ch = s[i];
        if ((ch >= 'A' && ch <= 'Z') || (ch >= 'a' && ch <= 'z') || (ch >= '0' && ch <= '9')) {
            if (needsp) {
                result += '-';
            }
            result += ch;
            needsp = false;
        } else {
            needsp = (result.size() != 0);
        }
    }
    return result;
}


string_t
makeGameUrl(int32_t gameId, string_t gameName)
{
    return GAME_BASE_URL + simplifyTopic(itoa(gameId) + "-" + gameName);
}

string_t
makeForumUrl(int32_t forumId, string_t forumName)
{
    return FORUM_BASE_URL + simplifyTopic(itoa(forumId) + "-" + forumName);
}

string_t
makePostUrl(int32_t topicId, string_t subject, int32_t messageId)
{
    return THREAD_BASE_URL + simplifyTopic(itoa(topicId) + "-" + subject) + "#p" + itoa(messageId);
}

string_t
makeTopicUrl(int32_t topicId, string_t subject)
{
    return THREAD_BASE_URL + simplifyTopic(itoa(topicId) + "-" + subject);
}
