/**
  *  \file planetscentral/talk/userfolder.h
  *  \brief Talk: User Folders (for PMs)
  */
#ifndef C2SERVER_PLANETSCENTRAL_TALK_USERFOLDER_H
#define C2SERVER_PLANETSCENTRAL_TALK_USERFOLDER_H

#include "planetscentral/dbclient/dbsubtree.h"
#include "cpluslib/types.h"
#include "cpluslib/string.h"

class User;

/** User folder. Stores personal messages (UserPM). The user's folders
    (headers and content) are managed in his user profile tree.
    As a special exception, there are default folders provided by the
    system. Default values for their headers are stored in the "default"
    tree. Users cannot delete those folders (although they can create
    their own headers for them).

    A folder is a set of messages. This means it cannot contain two
    copies of a message. */
class UserFolder {
 public:
    UserFolder(User& user, int32_t userFolderId);

    bool checkExistance();

    DbIntSet       messages();
    DbHash         header();
    DbHashIntField unreadMessages();

    string_t getHeader(string_t key);
    IntValue* describe(bool isUser);

    void remove();

    static int32_t allocateFolder(User& user);
    static DbIntSet defaultFolders(DbClient& client);

 private:
    DbHash defaultHeader();

    DbIntSet userFolderSet;
    DbSubtree userFolder;
    int32_t userFolderId;
};

#endif
